from nonconformist.base import ClassifierAdapter
from nonconformist.base import RegressorAdapter
import numpy as np

class MyClassifierAdapter(ClassifierAdapter):
    r"""Class to adapt classifiers to use them with the nonconformist package

    This class adapts the LGBMClassifier, XGBClassifier and CatBoostClassifier
    models, so they can be used with the nonconformist package. The model must
    be trained with the fit method.

    Examples
    --------
    >>> model = lightgbm.LGBMClassifier()
    >>> adapted_model = MyClassifierAdapter(model)
    """

    def __init__(self, model, fit_params=None):
        r""" __init__ method.

        This method is used for creating a MyClassifierAdapter object with
        a given `model`

        Parameters
        ----------
        model:
            Model we want to adapt to use with nonconformist.
        """
        super(MyClassifierAdapter, self).__init__(model, fit_params)

    def fit(self, x: np.ndarray, y: np.ndarray) -> None:
        r""" Method for fitting the underlying model.

        This function adapts the fitting method so `model` can be used as
        an underlying model with the nonconformist package.

        Parameters
        ----------
        x: numpy.ndarray
            Array of features used to fit the model
        y: numpy.ndarray
            Array of target fetures used to fit the model

        Returns
        -------
        None
        """
        self.model.fit(x, y)

    def predict(self, x: np.ndarray):
        r"""Method for generating predictions from the underlying model

        This function adapts the predicting method so the `model`can be used as an
        underlying model with the nonconformist package.

        Parameters
        ----------
        x: numpy.ndarray
            Array of features used to predict the target feature

        Returns
        -------
        model.predict_proba: np.ndarray
            Array of predicted features
        """
        return self.model.predict_proba(x)

class MyRegressorAdapter(RegressorAdapter):
    r"""Class to adapt regressors to use them with the nonconformist package

    This class adapts the LGBMRegressor, XGBRegressor and CatBoostRegressor
    models, so they can be used with the nonconformist package. The model used
    must be trained with the fit method.

    """

    def __init__(self, model, fit_params=None):
        r""" __init__ method.

        This method is used for creating a MyRegressorAdapter object with
        a given `model`

        Parameters
        ----------
        model:
            Model we want to adapt to use with nonconformist.
        """
        super(MyRegressorAdapter, self).__init__(model, fit_params)

    def fit(self, x: np.ndarray, y: np.ndarray) -> None:
        r""" Method for fitting the underlying model.

        This function adapts the fitting method so `model` can be used as
        an underlying model with the nonconformist package.

        Parameters
        ----------
        x: numpy.ndarray
            Array of features used to fit the model
        y: numpy.ndarray
            Array of target fetures used to fit the model

        Returns
        -------
        None
        """
        self.model.fit(x, y)

    def predict(self, x: np.ndarray):
        r"""Method for generating predictions from the underlying model

        This function adapts the predicting method so the `model`can be used as an
        underlying model with the nonconformist package.

        Parameters
        ----------
        x: numpy.ndarray
            Array of features used to predict the target feature

        Returns
        -------
        model.predict: np.ndarray
            Array of predicted features
        """
        return self.model.predict(x)

class MyPreTrainedRegressorAdapter(RegressorAdapter):
    r"""Class to adapt pre-trained regressors to use them with the nonconformist package

    This class adapts pre-trained LGBMRegressor models, which must be loaded with the
    lightgbm.Booster fuction.

    """

    def __init__(self, model, fit_params=None):
        r""" __init__ method.

        This method is used for creating a MyRegressorAdapter object with
        a given `model`

        Parameters
        ----------
        model:
            Pre-trained LGBMRegressor model we want to adapt to use with nonconformist.
        """
        super(MyRegressorAdapter, self).__init__(model, fit_params)

    def fit(self, x: np.ndarray, y: np.ndarray) -> None:
        r""" Method for fitting the underlying model.

        This function adapts the fitting method so `model` can be used as
        an underlying model with the nonconformist package. In this case, since
        the model has been fitted, the method does nothing.

        Parameters
        ----------
        x: numpy.ndarray
            Array of features used to fit the model
        y: numpy.ndarray
            Array of target fetures used to fit the model

        Returns
        -------
        None
        """
        pass
    
    def predict(self, x: np.ndarray):
        r"""Method for generating predictions from the underlying model

        This function adapts the predicting method so the `model`can be used as an
        underlying model with the nonconformist package.

        Parameters
        ----------
        x: numpy.ndarray
            Array of features used to predict the target feature

        Returns
        -------
        model.predict: np.ndarray
            Array of predicted features
        """
        return self.model.predict(x)



# Imports
import numpy as np
from nonconformist.cp import IcpRegressor
from nonconformist.cp import IcpClassifier
from nonconformist.nc import NcFactory
from nonconformist.base import ClassifierAdapter
from nonconformist.nc import ClassifierNc
from nonconformist.base import RegressorAdapter
from nonconformist.nc import RegressorNc
from sklearn.base import is_classifier, is_regressor
import lightgbm as lgbm
from .adapter_classes import MyClassifierAdapter, MyRegressorAdapter, MyPreTrainedRegressorAdapter

# non sklearn adapter funtion
def NonConformistAdapter(model):
    r""" Function for adapting non Scikit Learn model so they can be used with nonconformist

    This funcitons adapts a lightgbm, xgboost or catboost `model` in order to use it
    with the nonconformist package and create confidence intervals. It's compatible
    with: LGBMRegressor (this model can be trained with the nonconformist fit method
    or it can be a pre-trained model loaded using lightgbm.Booster), LGBMClassifier, 
    XGBRegressor, XGBClassifier, CatBoostRegressor and CatBoostClassifier (these must
    trained with the nonconformist fit method).

    Parameters
    ----------
    model:
        Model we want to adapt in order to use it with nonconformist.

    Returns
    -------
    adapted_model: obj: MyClassifierAdapter, MyRegressorAdapter or MyPreTrainedRegressorAdapter
        The class of the adapted model so it can be used to create conformal prediction intervals.

    Raises
    ------
    ValueError
        The model isn't one of the specified classes.

    Examples:
    --------
    >>> model = lightgbm.LGBMRegressor()
    >>> adapted_model = NonConformistAdapter(model)
    """
    classifiers = ["LGBMClassifier", "XGBClassifier", "CatBoostClassifier"]
    regressors = ["LGBMRegressor","XGBRegressor", "CatBoostRegressor"]
    cbr_quant_50 = "Booster"
    if model.__class__.__name__ in classifiers:
        return MyClassifierAdapter(model)
    elif model.__class__.__name__ in regressors:
        return MyRegressorAdapter(model)
    elif model.__class__.__name__ == cbr_quant_50:
        return MyPreTrainedRegressorAdapter(model)
    else:
        raise ValueError(f"Model type should be {classifiers} for classifiers or {regressors} or {cbr_quant_50} for regressors")

# Adapter class
class Adapt_to_CP():
    r""" Class to adapt models so the produce predictions with confidence intervals

    This class transforms sklearn, lightgbm, xgboost and catboost models so they can 
    perform conformal prediction. In order to initiate this class the `model` must be
    given as an input. In adition to the `model`, a boolean variable `sklearn_model`
    must be given, indicating if the model belongs to scikit learn (True) or
    not (False).
    """

    def __init__(self, model, sklearn_model: bool):
        r"""__init__ method

        This method is used to adapt the input `model` so it can be used for creating 
        confidente intervals with conformal prediction.

        Parameters
        ----------
        model:
            Model we want to use as the underlying model to generate predictions and the
            confidence interval. This model can only be a scikit learn model, LGBMRegressor,
            LGBMClassifier, XGBRegressor, XGBClassifier, CatBoostRegressor or CatBoostClassifier.
        sklearn_model: bool
            This variable indicates if the model belongs to scikit learn or not.

        Returns
        -------
        cp: obj: Adapt_to_CP
            The class of the adapted model.

        Examples
        --------
        >>> model = lightgbm.LGBMRegressor()
        >>> cp = Adapt_to_CP(model)
        """
        self.model = model
        if sklearn_model:
            if is_classifier(model):
                self.icp = IcpClassifier(NcFactory.create_nc(model))
            elif is_regressor(model):
                self.icp = IcpRegressor(NcFactory.create_nc(model))
        else:
            model_adapter = NonConformistAdapter(model)
            if is_classifier(model):
                self.icp = IcpClassifier(ClassifierNc(model_adapter))
            elif is_regressor(model):
                self.icp = IcpRegressor(RegressorNc(model_adapter))
            elif model.__class__.__name__ == "Booster":
                self.icp = IcpRegressor(RegressorNc(model_adapter))

    def fit(self, x_train: np.ndarray, y_train: np.ndarray):
        r"""Method used to fit the underlying model

        In order to create the prediction and the confidence interval the underlying
        model must be fitted first. This fuction trains the model using the data features
        `x_train` and the target features `y_train`.

        Parameters
        ----------
        x_train: numpy.ndarray
            Array of data fetures the model will be trained with.
        y_train: numpy.ndarray
            Array of target features the model is trained to predict.

        Returns
        -------
        None
        """
        self.icp.fit(x_train, y_train)

    def calibrate(self, x_cal: np.ndarray, y_cal: np.ndarray):
        r"""Method used to calculate the nonconformity scores

        To create the confidence intervals we need a set of nonconformity scores,
        and, in classification cases, their probability distribution. This method
        uses the fitted underlying model and calculates de scores, thus calibrating
        the inductive conformal predictor.

        Parameters
        ----------
        x_cal: numpy.ndarray
            Array of data features used for calibrating the inductive conformal predictor.
        y_cal: numpy.ndarray
            Array of target features used for calibrating the inductive conformal predictor

        Returns
        -------
        None

        Notes
        -----
        It's very important to make sure the calibration data is different from the training data.
        """
        self.icp.calibrate(x_cal, y_cal)

    def predict(self, x_test: np.ndarray, confidence: float):
        r"""Method that returns the prediction and the confidence interval

        This method returns the interval with a confidence level of `confidence` and the target
        predictions for `x_test`. The information returned for classification is different from the
        one returned for regression. In classification cases the tuple returned has two elements: a
        numpy.ndarray with a matrix of boolean values and a numpy.ndarray that contains the class
        predictions. On the other hand, in regression cases the tuple returned has 3 elements: a numpy.ndarray
        with the lower bound values, a numpy.ndarray with the predicted target values and a numpy.ndarray
        with the upper bound values.

        Parameters
        ----------
        x_test: numpy.ndarray
            Array of data features used to predict the target values and the confidence interval
        confidence: float
            Float between 0 and 1 that represent the percentage of observations we want to be
            inside the predicted interval.

        Returns
        -------
        prediction: Tuple[numpy.ndarray, numpy.ndarray] or Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray]
            Tuple containing the confidence interval and the target prediction

        Notes
        -----
        The `x_test` data must have the same features as the data used for training and calibration,
        and they must be in the same order.
        The level of confidence has to be a fraction between 0 and 1.
        """
        sig = 1 - confidence
        if is_classifier(self.model):
            return self.icp.predict(x_test, significance=sig), self.model.predict(x_test)
        elif is_regressor(self.model):
            return self.icp.predict(x_test, significance=sig)[:, 0], self.model.predict(x_test), self.icp.predict(x_test, significance=sig)[:, 1]
        elif type(self.model) == lgbm.basic.Booster:
            return self.icp.predict(x_test, significance=sig)[:, 0], self.model.predict(x_test), self.icp.predict(x_test, significance=sig)[:, 1]

    def calibrate_and_predict(self, x_cal: np.ndarray, y_cal: np.ndarray, x_test: np.ndarray, confidence: bool):
        r"""Method used for calibrating the conformal predictor and predicting target values and the confidence interval

        This method is equal to running the calibrate and the predict methods consecutively. 
        Accordingly, the inputs are the calibration data `x_cal` and targets `y_cal`, the test data `x_test` and the
        confidence level `confidence` desired for the predicted interval. The tuple returned contains
        the predicted values and the confidence interval.

        Parameters
        ----------
        x_cal: numpy.ndarray
            Array of data features used for calibrating the inductive conformal predictor.
        y_cal: numpy.ndarray
            Array of target features used for calibrating the inductive conformal predictor
        x_test: numpy.ndarray
            Array of data features used to predict the target values and the confidence interval
        confidence: float
            Float between 0 and 1 that represent the percentage of observations we want to be
            inside the predicted interval.

        Returns
        -------
        prediction: Tuple[numpy.ndarray, numpy.ndarray] or Tuple[numpy.ndarray, numpy.ndarray, numpy.ndarray]
            Tuple containing the confidence interval and the target prediction

        Notes
        -----
        Both the calibration and the test data must have the same features (scale and order) as the data
        used for training the underlying model.
        The level of confidence has to be a decimal between 0 and 1.
        """
        sig = 1 - confidence
        self.icp.calibrate(x_cal, y_cal)
        if is_classifier(self.model):
            return self.icp.predict(x_test, significance=sig), self.model.predict(x_test)
        elif is_regressor(self.model):
            return self.icp.predict(x_test, significance=sig)[:, 0], self.model.predict(x_test), self.icp.predict(x_test, significance=sig)[:, 1]
        elif type(self.model) == lgbm.basic.Booster:
            return self.icp.predict(x_test, significance=sig)[:, 0], self.model.predict(x_test), self.icp.predict(x_test, significance=sig)[:, 1]



import numpy as np
import pandas as pd
from typing import Tuple

def train_cal_test_split(df: pd.DataFrame,
                         target_column_name: str,
                         train_percentage: float,
                         cal_percentage: float,
                         random: bool) -> Tuple[np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray, np.ndarray]:
    r"""Function used for splitting the dataset in training, calibration and test subsets

    This fuction splits the data in `df` in three sets acording to the percentajes given as input:
    `train_percentage` and `cal_percentage`. The Tuple returned has 6 elements: the training data, the training
    target values, the calibration data, the calibration target values, the test data and target values.

    Parameters
    ----------
    df: pandas.DataFrame
        Data we want to split in training, calibration and test sets
    target_columns_name: str
        The name of the target feature
    train_percentage: float
        Fraction of the data we want to use for the training process
    cal_percentage: float
        Fraction of the data we want to use to calibrate the inductive
        conformal predictor
    random: bool
        Variable that defines if the dataset is shuffled before the split.
        True -> shuffle and False -> don't shuffle.

    Returns
    -------
    split_data: Tuple[ndarray, ndarray, ndarray, ndarray, ndarray, ndarray]
        Tuple with 6 elements: the trainign data, the training target feature,
        the calibration data and target, the test data and, finally, the test target values.

    Notes
    -----
    This fuction only works when there's only one target feature.
    """
    if random:
        df = df.sample(frac=1).reset_index(drop=True)
    X = df.drop(columns=target_column_name).to_numpy()
    Y = df[target_column_name].to_numpy()
    n_total = X.shape[0]
    n_train = int(train_percentage*n_total)
    n_cal = int(cal_percentage*n_total) + n_train
    train_data = X[:n_train, :]
    train_target = Y[:n_train]
    cal_data = X[n_train:n_cal, :]
    cal_target = Y[n_train:n_cal]
    test_data = X[n_cal:, :]
    test_target = Y[n_cal:]
    return train_data, train_target, cal_data, cal_target, test_data, test_target


import numpy as np


def right_guess(y: np.ndarray, 
                preds: np.ndarray) -> float:
     r"""Function used to calculate the coverage of a classification interval

     This function return the fraction of times the confidence interval
     `preds` contains the classes in `y`.

     Parameters
     ----------
     y: numpy.ndarray
          Array of real or predicted classes with length num_samples
     preds: numpy.ndarray
          Array containing the confidence interval predicted for classification cases.
          This type of interval is a boolean matrix with size [num_samples, num_classes],
          where num_classes is the total number of possible classes for the data.
     
     Returns
     -------
     coverage: float
          Fraction of times the classification in `y` are contained in the predicted confidence
          interval.

     Notes
     -----
     This fuction only works with classification cases, where the confidence interval is created
     using the nonconformist package.
     """
     n_samples = len(y)
     count = np.sum(preds[range(len(preds)), y])
     return count/n_samples


def uncertainty(preds: np.ndarray) -> float:
     r"""Function used to calculate the fraction of uncertainty of the predicted confidence interval

     Sometimes the nonconformist inductive conformal predictor returns an interval that contains all
     the possile classes. In these cases no information is provided, thus adding uncertainty to the
     prediction. This function returns the fraction of times the interval created has all the classes.

     Parameters
     ----------
     preds: numpy.ndarray
          The confidence interval, represented by a numpy.ndarray of boolean
          values with size (num_samples, num_classes), where num_classes is the total
          number of classes for the data, and num_samples is the number of samples used
          to create the matrix.

     Returns
     -------
     uncertainty: float
          Number of times the interval created contains all the possible classes.

     Notes
     -----
     This fuction can only be used in confidence interval generated for classification
     cases with the nonconformist package.
     """
     return np.mean(preds.all(axis=1))


def width(preds: np.ndarray) -> float:
     r"""Function that calculates the normalized width of a classification confidence interval

     This measure represents the with of the classification interval `preds`.
     It's calculated as the average number classes in the predicted intervals for
     every sample, dived by the total number of clases.

     Parameters
     ----------
     preds: numpy.ndarray
          The confidence interval, represented by a numpy.ndarray of boolean
     values with size (num_samples, num_classes), where num_classes is the total
     number of classes for the data, and num_samples is the number of samples used
     to create the matrix.

     Returns
     -------
     mean_norm: float
          Normalized width of a classification confidence interval

     Notes
     -----
     This function can only be used in classification confidence intervals created with
     the nonconformist package.
     """
     n_classes = preds.shape[1]
     n_samples = preds.shape[0]
     n_trues = np.sum(preds)
     mean = n_trues / n_samples
     mean_norm = mean / n_classes
     return mean_norm


def picp(real: np.ndarray, lower: np.ndarray, upper: np.ndarray) -> float:
     r"""Function that calculates the coverage of the predicted interval

     The PICP (Prediction Interval Coverage Probability) measures the
     fraction of times the prediction or real data in `real` is above
     the lower bound `lower` and below the upper bound `upper` of the predicted
     confidence interval.

     Parameters
     ----------
     real: numpy.ndarray
          Array of the predicted or real target values.
     lower: numpy.ndarray
          Array of predicted lower bound values.
     upper: numpy.ndarray
          Array of predited upper bound values.

     Returns
     -------
     picp: float
          Fraction of samples that fall inside the predicted interval.

     Notes
     -----
     This measure is useful only for regression confidence intervals created
     with the nonconformist package.

     """
     return ((real <= upper) & (real >= lower)).mean()


def pinaw(real: np.ndarray, lower: np.ndarray, upper: np.ndarray) -> float:
     r"""Function for calulating the normalized average width of a predicted interval

     The PINAW (Prediction Interval Normalized Width) represents the average
     interval width, dived by the range of the data (max value - min value). This 
     fuction calculates de PINAW for a regression interval.

     Parameters
     ----------
     real: numpy.ndarray
          Array of the predicted or real target values.
     lower: numpy.ndarray
          Array of predicted lower bound values.
     upper: numpy.ndarray
          Array of predited upper bound values.

     Returns
     -------
     pinaw: float
          The normalized average width of the predicted interval

     Notes
     -----
     This measure is only useful when the interval is predicted for a
     regression case.
     """
     range = real.max() - real.min()
     width = upper - lower
     return (width.mean()/range)


def relative_width(real: np.ndarray, lower: np.ndarray, upper: np.ndarray) -> float:
     r"""Fuction used fo calculating the average width relative to the value of the target features

     The measure given by this fuction is calculated as the median value of the interval
     width diveded by the target values in `real`.

     Parameters
     ----------
     real: numpy.ndarray
          Array of the predicted or real target values.
     lower: numpy.ndarray
          Array of predicted lower bound values.
     upper: numpy.ndarray
          Array of predited upper bound values.

     Returns
     -------
     median_relative_width: float
          Median value of the width divided by the target feature value.

     Notes
     -----
     This measure is useful for regression cases, not classification.
     """
     width = upper - lower
     return np.median(width/real)


def relative_mean_width(real: np.ndarray, lower: np.ndarray, upper: np.ndarray) -> float:
     r"""Fuction used for calculating the average width relative to the mean value of the target feature.

     The measure returned by this function is calculated as the mean interval width divided by
     the mean target value in `real`.

     Parameters
     ----------
     real: numpy.ndarray
          Array of the predicted or real target values.
     lower: numpy.ndarray
          Array of predicted lower bound values.
     upper: numpy.ndarray
          Array of predited upper bound values.

     Returns
     -------
     relative_mean_width: float
          Average interval width divided by the mean value of the target feature.

     Notes
     -----
     This measure is used for interval created for regression cases
     """
     width = upper - lower
     mean_real = real.mean()
     return width.mean()/mean_real


import numpy as np
import matplotlib.pyplot as plt
import seaborn
from typing import List

def band_interval_plot(y: np.ndarray, lower: np.ndarray, upper: np.ndarray, conf_percentage: float, sort: bool) -> None:
	r"""Function used to plot the data in `y` and it's confidence interval 

	This function plots `y`, with a line plot, and the interval defined by the
	`lower` and `upper` bounds, with a band plot.

	Parameters
	----------
	y: numpy.ndarray
		Array of observation or predictions we want to plot
	lower: numpy.ndarray
		Array of lower bound predictions for the confidence interval
	upper: numpy.ndarray
		Array of upper bound predictions for the confidence interval
	conf_percetage: float
		The desired confidence level of the predicted confidente interval
	sort: bool
		Boolean variable that indicates if the data, and the respective lower
		and upper bound values should be sorted ascendingly.

	Returns
	-------
	None

	Notes
	-----
	This function must only be used for regression cases
	"""
	if sort:
		idx = np.argsort(y)
		y = y[idx]
		lower = lower[idx]
		upper = upper[idx]
	fig, ax = plt.subplots(figsize=(14, 8))
	ax.plot(y.reshape(-1), label='data')
	conf = str(conf_percentage*100) + '%'
	ax.fill_between([*range(y.shape[0])], lower, upper, label=conf, alpha=0.3)
	ax.legend()

def conditional_band_interval_plot(y, lower, upper, sort):
	r"""Function for plotting the data in `y` and it's confidence interval

	This function plots `y` and it's confidence interval. The confidence
	interval is green when the `y` observation falls inside the predicted
	interval, on the other hand, it's red when it doesn't. The interval is
	defined by the `lower` and `upper` bound.

	Parameters
	----------
	y: numpy.ndarray
		Array of observation or predictions we want to plot
	lower: numpy.ndarray
		Array of lower bound predictions for the confidence interval
	upper: numpy.ndarray
		Array of upper bound predictions for the confidence interval
	conf_percetage: float
		The desired confidence level of the predicted confidente interval
	sort: bool
		Boolean variable that indicates if the data, and the respective lower
		and upper bound values should be sorted ascendingly.

	Returns
	-------
	None

	Notes
	-----
	This function must only be used for regression cases
	"""
	if sort:
		idx = np.argsort(y)
		y = y[idx]
		lower = lower[idx]
		upper = upper[idx]
	fig, ax = plt.subplots(figsize=(14, 8))
	ax.plot(y.reshape(-1), 'o', label='data')
	ax.fill_between([*range(y.shape[0])], lower, upper, label='target inside the interval', 
					alpha=0.3, color='g', where=[l <= y_i and y_i <= u for (l, y_i, u) in zip(lower, y, upper)])
	ax.fill_between([*range(y.shape[0])], lower, upper, label='target outside the interval',
					alpha=0.3, color='r', where=[l >= y_i or y_i >= u for (l, y_i, u) in zip(lower, y, upper)])
	ax.legend()

def line_interval_plot(y, lower, upper, sort):
	r"""Function for plotting `y` and it's confidence interval

	The `y` data is represented by a big blue point. On the other had, 
	the `lower` bound of the interval is represented by a smaller red point
	and the `upper` bound by a smmaler blue point, both bounds are connected
	by a vertical line.

	Parameters
	----------
	y: numpy.ndarray
		Array of observation or predictions we want to plot
	lower: numpy.ndarray
		Array of lower bound predictions for the confidence interval
	upper: numpy.ndarray
		Array of upper bound predictions for the confidence interval
	conf_percetage: float
		The desired confidence level of the predicted confidente interval
	sort: bool
		Boolean variable that indicates if the data, and the respective lower
		and upper bound values should be sorted ascendingly.

	Returns
	-------
	None

	Notes
	-----
	This function must only be used for regression cases
	"""
	if sort:
		idx = np.argsort(y)
		y = y[idx]
		lower = lower[idx]
		upper = upper[idx]
	fig, ax = plt.subplots(figsize=(14, 8))
	ax.plot(y.reshape(-1), 'o',label='data')
	x = np.arange(y.shape[0])
	ax.vlines(x, ymin=lower, ymax=upper)
	ax.plot(lower, 'ro', markersize = 4, label='lower limit')
	ax.plot(upper, 'bo', markersize = 4, label='upper limit')
	ax.legend()

def classes_in_interval(preds):
	classes = []
	for i in range(preds.shape[0]):
		classes.append(np.where(preds[i, :]))
	return classes

def plot_class_interval(y: np.ndarray, preds: np.ndarray) -> None:
	r"""Function thas shows the classes in `y` and the classes in it's confidence interval.

	The class for every observation in `y` is represented by a white dot,
	while the classes in the interval `preds` can be seen as a red *.
	When the "y" class and one of the classes in the interval overlap, the
	white dot turns red.

	Parameters
	----------
	y: numpy.ndarray
		Array of predicted or real classes of length num_samples
	preds: numpy.ndarray
		The confidence interval, represented by a numpy array of boolean
		values with size (num_samples, num_classes), where num_classes is the total
		number of classes for the data.

	Returns
	-------
	None

	Notes
	-----
	This function must be used in classification cases
	"""
	x = np.arange(len(y))
	pred_classes = classes_in_interval(preds)
	fig, ax = plt.subplots(figsize=(14, 8))
	ax.scatter(x, y, edgecolors='black', color='w')
	for xe, ye in zip(x, pred_classes):
		ye = np.asarray(ye).reshape(-1, 1)
		ax.scatter([xe] * len(ye), ye, alpha=0.5, color='r', marker='*')
	ax.legend(['class', 'classes in predicted interval'])


def class_histogram(y: np.ndarray, preds: np.ndarray) -> None:
	r"""Function used to plot the histogram of classes in `y` and the classes in it's confidence interval

	There are two histograms in the plot shown by this fuction, the red one corresponds to the
	class histogram of the input `y`. The blue one is the total class histogram of the predicted
	confidence interval `preds`.

	Parameters
	----------
	y: numpy.ndarray
		Array containing the predicted or real classes in num_samples
	preds: numpy.ndarray
		The confidence interval, represented by a numpy array of boolean
		values with size (num_samples, num_classes), where num_classes is the total
		number of classes for the data.

	Returns
	-------
	None

	Notes
	-----
	This function must be used in classification cases
	"""
	pred_classes = np.array(classes_in_interval(preds))
	interval =  np.concatenate(np.concatenate(pred_classes, axis=0), axis=0)
	fig, ax = plt.subplots(figsize=(14, 8))
	ax.hist(interval, alpha=0.3)
	ax.hist(y, alpha=0.4)
	ax.legend(['classes in intervals', 'classes'])

def confusion_matrix(y: np.ndarray, pred: np.ndarray, classes: List[str]) -> None:
	r"""Function for displaying the cofusion matrix

	This function shows a confusion matrix for the real classes in `y`
	and the classes in the predicted interval `preds`. In this case the numbers
	inside the matrix represent the fraction of times a certain class (x axis) is present
	in the confidence interval, when the real class was the one on the y axis.

	Parameters
	----------
	y: numpy.ndarray
		Array of predicted or real classes
	preds: numpy.ndarray
		The confidence interval, represented by a numpy array of boolean
		values with size (num_samples, num_classes), where num_classes is the total
		number of classes for the data.
	classes: List[str]
		List of the classes' names.

	Returns
	-------
	None

	Notes
	-----
	This function must be used in classification cases
	"""
	data = []
	for i in range(len(classes)):
		idx = np.where(y == i)[0]
		class_count = pred[idx].sum(axis=0)
		class_count_norm = class_count/len(idx)
		data.append(class_count_norm.tolist())
	seaborn.set(color_codes=True)
	plt.figure(1, figsize=(18,  18))
	plt.title("Confusion Matrix")
	seaborn.set(font_scale=1)
	ax = seaborn.heatmap(data, annot=True, cmap="YlGnBu", cbar_kws={'label': 'Scale'})
	ax.set_xticklabels(classes)
	ax.set_yticklabels(classes)
	ax.set(ylabel="True Label", xlabel="Predicted Label in Interval")
	plt.show()
	plt.close()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cpadapter']

package_data = \
{'': ['*']}

install_requires = \
['lightgbm>=3.1,<4.0',
 'matplotlib>=3.0,<4.0',
 'nonconformist>=2.1.0,<3.0.0',
 'numpy>=1.19,<2.0',
 'pandas>=1.1,<2.0',
 'seaborn>=0.11,<0.12',
 'sklearn>=0.0,<0.1',
 'typing>=3.7,<4.0']

setup_kwargs = {
    'name': 'cpadapter',
    'version': '0.1.1',
    'description': 'This package adapts different models in order to create confidence intervals using conformal prediction',
    'long_description': None,
    'author': 'mjesusugarte',
    'author_email': 'maria.jesus@spikelab.xyz',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
